@@ -25,7 +25,7 @@
 #include <boost/math/tools/assert.hpp>
 #include <boost/math/tools/config.hpp>
 
-#ifdef BOOST_HAS_THREADS
+#ifdef BOOST_MATH_HAS_THREADS
 #include <mutex>
 #endif
 
@@ -53,7 +53,7 @@ namespace boost { namespace math { namespace detail{
         if(n == 1) return 1 / x;
         T nlx = n * log(x);
         if((nlx < tools::log_max_value<T>()) && (n < (int)max_factorial<T>::value))
-           return ((n & 1) ? 1 : -1) * boost::math::factorial<T>(n - 1, pol) * pow(x, -n);
+           return ((n & 1) ? 1 : -1) * boost::math::factorial<T>(n - 1, pol) * static_cast<T>(pow(x, T(-n)));
         else
          return ((n & 1) ? 1 : -1) * exp(boost::math::lgamma(T(n), pol) - n * log(x));
      }
@@ -65,12 +65,12 @@ namespace boost { namespace math { namespace detail{
      // (n-1)! / x^(n+1)
      //
      // which is common to both the first term of the series (with k = 1)
-     // and to the leading part.  
+     // and to the leading part.
      // We can then get to the leading term by:
      //
      // part_term * (n + 2 * x) / 2
      //
-     // and to the first term in the series 
+     // and to the first term in the series
      // (excluding the Bernoulli number) by:
      //
      // part_term n * (n + 1) / (2x)
@@ -79,13 +79,13 @@ namespace boost { namespace math { namespace detail{
      // or the power term underflows, this just gets set to 0 and then we
      // know that we have to use logs for the initial terms:
      //
-     part_term = ((n > (int)boost::math::max_factorial<T>::value) && (T(n) * n > tools::log_max_value<T>())) 
-        ? T(0) : static_cast<T>(boost::math::factorial<T>(n - 1, pol) * pow(x, -n - 1));
+     part_term = ((n > (int)boost::math::max_factorial<T>::value) && (T(n) * n > tools::log_max_value<T>()))
+        ? T(0) : static_cast<T>(boost::math::factorial<T>(n - 1, pol) * pow(x, T(-n - 1)));
      if(part_term == 0)
      {
         // Either n is very large, or the power term underflows,
         // set the initial values of part_term, term and sum via logs:
-        part_term = static_cast<T>(boost::math::lgamma(n, pol) - (n + 1) * log(x));
+        part_term = static_cast<T>(T(boost::math::lgamma(n, pol)) - (n + 1) * log(x));
         sum = exp(part_term + log(n + 2 * x) - boost::math::constants::ln_two<T>());
         part_term += log(T(n) * (n + 1)) - boost::math::constants::ln_two<T>() - log(x);
         part_term = exp(part_term);
@@ -126,7 +126,7 @@ namespace boost { namespace math { namespace detail{
            return policies::raise_evaluation_error(function, "Series did not converge, closest value was %1%", sum, pol);
         }
      }
-     
+
      if((n - 1) & 1)
         sum = -sum;
 
@@ -159,7 +159,7 @@ namespace boost { namespace math { namespace detail{
     {
        for(int k = 1; k <= iter; ++k)
        {
-          z_plus_k_pow_minus_m_minus_one = pow(z, minus_m_minus_one);
+          z_plus_k_pow_minus_m_minus_one = static_cast<T>(pow(z, T(minus_m_minus_one)));
           sum0 += z_plus_k_pow_minus_m_minus_one;
           z += 1;
        }
@@ -200,22 +200,22 @@ namespace boost { namespace math { namespace detail{
      T factorial_part = 1;
      //
      // "prefix" is what we'll be adding the accumulated sum to, it will
-     // be n! / z^(n+1), but since we're scaling by n! it's just 
+     // be n! / z^(n+1), but since we're scaling by n! it's just
      // 1 / z^(n+1) for now:
      //
-     T prefix = pow(x, n + 1);
+     T prefix = static_cast<T>(pow(x, T(n + 1)));  // Warning supression: Integer power returns at least a double
      if(prefix == 0)
-        return boost::math::policies::raise_overflow_error<T>(function, 0, pol);
+        return boost::math::policies::raise_overflow_error<T>(function, nullptr, pol);
      prefix = 1 / prefix;
      //
      // First term in the series is necessarily < zeta(2) < 2, so
      // ignore the sum if it will have no effect on the result anyway:
      //
      if(prefix > 2 / policies::get_epsilon<T, Policy>())
-        return ((n & 1) ? 1 : -1) * 
-         (tools::max_value<T>() / prefix < scale ? policies::raise_overflow_error<T>(function, 0, pol) : prefix * scale);
+        return ((n & 1) ? 1 : -1) *
+         (tools::max_value<T>() / prefix < scale ? policies::raise_overflow_error<T>(function, nullptr, pol) : prefix * scale);
      //
-     // As this is an alternating series we could accelerate it using 
+     // As this is an alternating series we could accelerate it using
      // "Convergence Acceleration of Alternating Series",
      // Henri Cohen, Fernando Rodriguez Villegas, and Don Zagier, Experimental Mathematics, 1999.
      // In practice however, it appears not to make any difference to the number of terms
@@ -245,7 +245,7 @@ namespace boost { namespace math { namespace detail{
      // We need to multiply by the scale, at each stage checking for overflow:
      //
      if(boost::math::tools::max_value<T>() / scale < sum)
-        return boost::math::policies::raise_overflow_error<T>(function, 0, pol);
+        return boost::math::policies::raise_overflow_error<T>(function, nullptr, pol);
      sum *= scale;
      return n & 1 ? sum : T(-sum);
   }
@@ -398,7 +398,7 @@ namespace boost { namespace math { namespace detail{
      }
 
      //
-     // We'll have to compute the coefficients up to n, 
+     // We'll have to compute the coefficients up to n,
      // complexity is O(n^2) which we don't worry about for now
      // as the values are computed once and then cached.
      // However, if the final evaluation would have too many
@@ -406,7 +406,7 @@ namespace boost { namespace math { namespace detail{
      //
      if((unsigned)n / 2u > policies::get_max_series_iterations<Policy>())
         return policies::raise_evaluation_error<T>(function, "The value of n is so large that we're unable to compute the result in reasonable time, best guess is %1%", 0, pol);
-#ifdef BOOST_HAS_THREADS
+#ifdef BOOST_MATH_HAS_THREADS
      static std::mutex m;
      std::lock_guard<std::mutex> l(m);
 #endif
@@ -459,45 +459,23 @@ namespace boost { namespace math { namespace detail{
      //
      T power_terms = n * log(boost::math::constants::pi<T>());
      if(s == 0)
-        return sum * boost::math::policies::raise_overflow_error<T>(function, 0, pol);
+        return sum * boost::math::policies::raise_overflow_error<T>(function, nullptr, pol);
      power_terms -= log(fabs(s)) * (n + 1);
      power_terms += boost::math::lgamma(T(n), pol);
      power_terms += log(fabs(sum));
 
      if(power_terms > boost::math::tools::log_max_value<T>())
-        return sum * boost::math::policies::raise_overflow_error<T>(function, 0, pol);
+        return sum * boost::math::policies::raise_overflow_error<T>(function, nullptr, pol);
 
      return exp(power_terms) * ((s < 0) && ((n + 1) & 1) ? -1 : 1) * boost::math::sign(sum);
   }
 
-  template <class T, class Policy>
-  struct polygamma_initializer
-  {
-     struct init
-     {
-        init()
-        {
-           // Forces initialization of our table of coefficients and mutex:
-           boost::math::polygamma(30, T(-2.5f), Policy());
-        }
-        void force_instantiate()const{}
-     };
-     static const init initializer;
-     static void force_instantiate()
-     {
-        initializer.force_instantiate();
-     }
-  };
-
-  template <class T, class Policy>
-  const typename polygamma_initializer<T, Policy>::init polygamma_initializer<T, Policy>::initializer;
-  
   template<class T, class Policy>
   inline T polygamma_imp(const int n, T x, const Policy &pol)
   {
     BOOST_MATH_STD_USING
     static const char* function = "boost::math::polygamma<%1%>(int, %1%)";
-    polygamma_initializer<T, Policy>::initializer.force_instantiate();
+
     if(n < 0)
        return policies::raise_domain_error<T>(function, "Order must be >= 0, but got %1%", static_cast<T>(n), pol);
     if(x < 0)
@@ -508,7 +486,7 @@ namespace boost { namespace math { namespace detail{
           // Result is infinity if x is odd, and a pole error if x is even.
           //
           if(lltrunc(x) & 1)
-             return policies::raise_overflow_error<T>(function, 0, pol);
+             return policies::raise_overflow_error<T>(function, nullptr, pol);
           else
              return policies::raise_pole_error<T>(function, "Evaluation at negative integer %1%", x, pol);
        }
@@ -540,7 +518,7 @@ namespace boost { namespace math { namespace detail{
     {
        T result = (n & 1 ? 1 : -1) * boost::math::factorial<T>(n, pol) * boost::math::zeta(T(n + 1), pol);
        if(fabs(result) >= ldexp(tools::max_value<T>(), -n - 1))
-          return boost::math::sign(result) * policies::raise_overflow_error<T>(function, 0, pol);
+          return boost::math::sign(result) * policies::raise_overflow_error<T>(function, nullptr, pol);
        result *= ldexp(T(1), n + 1) - 1;
        return result;
     }
