@@ -1,29 +1,27 @@
 ///////////////////////////////////////////////////////////////////////////////
-//  Copyright 2018 John Maddock. Distributed under the Boost
+//  Copyright 2018 - 2025 John Maddock.
+//  Copyright 2025 Christopher Kormanyos.
+//  Distributed under the Boost
 //  Software License, Version 1.0. (See accompanying file
 //  LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
-#ifndef BOOST_MULTIPRECISION_COMPLEX_ADAPTOR_HPP
-#define BOOST_MULTIPRECISION_COMPLEX_ADAPTOR_HPP
+#ifndef BOOST_MP_COMPLEX_ADAPTOR_HPP
+#define BOOST_MP_COMPLEX_ADAPTOR_HPP
 
 #include <boost/multiprecision/number.hpp>
-#include <cstdint>
 #include <boost/multiprecision/detail/digits.hpp>
 #include <boost/multiprecision/detail/hash.hpp>
-#include <cmath>
+#include <boost/multiprecision/detail/no_exceptions_support.hpp>
+
 #include <algorithm>
+#include <cmath>
 #include <complex>
+#include <cstdint>
 
 namespace boost {
 namespace multiprecision {
 namespace backends {
 
-template <class Backend>
-struct debug_adaptor;
-
-template <class Backend>
-struct logged_adaptor;
-
 template <class Backend>
 struct complex_adaptor
 {
@@ -62,6 +60,11 @@ struct complex_adaptor
        : m_real(val)
    {}
 
+   template <class T>
+   complex_adaptor(const T& val, const typename std::enable_if<std::is_convertible<T, Backend>::value>::type* = nullptr)
+       : m_real(val) 
+   {}
+
    complex_adaptor(const std::complex<float>& val)
    {
       m_real = (long double)val.real();
@@ -104,7 +107,7 @@ struct complex_adaptor
       return *this;
    }
    template <class V>
-   complex_adaptor& operator=(const V& v)
+   typename std::enable_if<std::is_assignable<Backend, V>::value, complex_adaptor&>::type operator=(const V& v)
    {
       using ui_type = typename std::tuple_element<0, unsigned_types>::type;
       m_real = v;
@@ -151,7 +154,7 @@ struct complex_adaptor
          else
             imag_data() = zero;
 
-         if (eval_fpclassify(imag_data()) == (int)FP_NAN)
+         if (eval_fpclassify(imag_data()) == static_cast<int>(FP_NAN))
          {
             real_data() = imag_data();
          }
@@ -192,6 +195,56 @@ struct complex_adaptor
       m_real.negate();
       m_imag.negate();
    }
+
+   //
+   // Default precision:
+   //
+   static BOOST_MP_CXX14_CONSTEXPR unsigned default_precision() noexcept
+   {
+      return Backend::default_precision();
+   }
+   static BOOST_MP_CXX14_CONSTEXPR void default_precision(unsigned digits10)
+   {
+      Backend::default_precision(digits10);
+      Backend::thread_default_precision(digits10);
+   }
+   static BOOST_MP_CXX14_CONSTEXPR unsigned thread_default_precision() noexcept
+   {
+      return Backend::thread_default_precision();
+   }
+   static BOOST_MP_CXX14_CONSTEXPR void thread_default_precision(unsigned digits10)
+   {
+      Backend::thread_default_precision(digits10);
+   }
+   BOOST_MP_CXX14_CONSTEXPR unsigned precision() const noexcept
+   {
+      return m_real.precision();
+   }
+   BOOST_MP_CXX14_CONSTEXPR void precision(unsigned digits10)
+   {
+      m_real.precision(digits10);
+      m_imag.precision(digits10);
+   }
+   //
+   // Variable precision options:
+   // 
+   static constexpr variable_precision_options default_variable_precision_options()noexcept
+   {
+      return Backend::default_variable_precision_options();
+   }
+   static constexpr variable_precision_options thread_default_variable_precision_options()noexcept
+   {
+      return Backend::thread_default_variable_precision_options();
+   }
+   static BOOST_MP_CXX14_CONSTEXPR void default_variable_precision_options(variable_precision_options opts)
+   {
+      Backend::default_variable_precision_options(opts);
+      Backend::thread_default_variable_precision_options(opts);
+   }
+   static BOOST_MP_CXX14_CONSTEXPR void thread_default_variable_precision_options(variable_precision_options opts)
+   {
+      Backend::thread_default_variable_precision_options(opts);
+   }
 };
 
 template <class Backend, class T>
@@ -237,48 +290,83 @@ inline void eval_divide(complex_adaptor<Backend>& result, const complex_adaptor<
    using default_ops::eval_subtract;
    Backend t1, t2;
 
+   //
+   // Backup sign bits for later, so we can fix up
+   // signed zeros at the end:
+   //
+   int a_sign = eval_signbit(result.real_data());
+   int b_sign = eval_signbit(result.imag_data());
+   int c_sign = eval_signbit(z.real_data());
+   int d_sign = eval_signbit(z.imag_data());
+
    if (eval_is_zero(z.imag_data()))
    {
       eval_divide(result.real_data(), z.real_data());
       eval_divide(result.imag_data(), z.real_data());
-      return;
-   }
-
-   eval_fabs(t1, z.real_data());
-   eval_fabs(t2, z.imag_data());
-   if (t1.compare(t2) < 0)
-   {
-      eval_divide(t1, z.real_data(), z.imag_data()); // t1 = c/d
-      eval_multiply(t2, z.real_data(), t1);
-      eval_add(t2, z.imag_data()); // denom = c * (c/d) + d
-      Backend t_real(result.real_data());
-      // real = (a * (c/d) + b) / (denom)
-      eval_multiply(result.real_data(), t1);
-      eval_add(result.real_data(), result.imag_data());
-      eval_divide(result.real_data(), t2);
-      // imag = (b * c/d - a) / denom
-      eval_multiply(result.imag_data(), t1);
-      eval_subtract(result.imag_data(), t_real);
-      eval_divide(result.imag_data(), t2);
    }
    else
    {
-      eval_divide(t1, z.imag_data(), z.real_data()); // t1 = d/c
-      eval_multiply(t2, z.imag_data(), t1);
-      eval_add(t2, z.real_data()); // denom = d * d/c + c
+      eval_fabs(t1, z.real_data());
+      eval_fabs(t2, z.imag_data());
+      if (t1.compare(t2) < 0)
+      {
+         eval_divide(t1, z.real_data(), z.imag_data()); // t1 = c/d
+         eval_multiply(t2, z.real_data(), t1);
+         eval_add(t2, z.imag_data()); // denom = c * (c/d) + d
+         Backend t_real(result.real_data());
+         // real = (a * (c/d) + b) / (denom)
+         eval_multiply(result.real_data(), t1);
+         eval_add(result.real_data(), result.imag_data());
+         eval_divide(result.real_data(), t2);
+         // imag = (b * c/d - a) / denom
+         eval_multiply(result.imag_data(), t1);
+         eval_subtract(result.imag_data(), t_real);
+         eval_divide(result.imag_data(), t2);
+      }
+      else
+      {
+         eval_divide(t1, z.imag_data(), z.real_data()); // t1 = d/c
+         eval_multiply(t2, z.imag_data(), t1);
+         eval_add(t2, z.real_data()); // denom = d * d/c + c
 
-      Backend r_t(result.real_data());
-      Backend i_t(result.imag_data());
+         Backend r_t(result.real_data());
+         Backend i_t(result.imag_data());
 
-      // real = (b * d/c + a) / denom
-      eval_multiply(result.real_data(), result.imag_data(), t1);
-      eval_add(result.real_data(), r_t);
-      eval_divide(result.real_data(), t2);
-      // imag = (-a * d/c + b) / denom
-      eval_multiply(result.imag_data(), r_t, t1);
-      result.imag_data().negate();
-      eval_add(result.imag_data(), i_t);
-      eval_divide(result.imag_data(), t2);
+         // real = (b * d/c + a) / denom
+         eval_multiply(result.real_data(), result.imag_data(), t1);
+         eval_add(result.real_data(), r_t);
+         eval_divide(result.real_data(), t2);
+         // imag = (-a * d/c + b) / denom
+         eval_multiply(result.imag_data(), r_t, t1);
+         result.imag_data().negate();
+         eval_add(result.imag_data(), i_t);
+         eval_divide(result.imag_data(), t2);
+      }
+   }
+   //
+   // Finish off by fixing up signed zeros.
+   // 
+   // This sets the signs "as if" we had evaluated the result using:
+   // 
+   // real = (ac + bd) / (c^2 + d^2)
+   // imag = (bc - ad) / (c^2 + d^2)
+   // 
+   // ie a zero is negative only if the two parts of the numerator
+   // are both negative and zero.
+   //
+   if (eval_is_zero(result.real_data()))
+   {
+      int r_sign = eval_signbit(result.real_data());
+      int r_required = (a_sign != c_sign) && (b_sign != d_sign);
+      if (r_required != r_sign)
+         result.real_data().negate();
+   }
+   if (eval_is_zero(result.imag_data()))
+   {
+      int i_sign = eval_signbit(result.imag_data());
+      int i_required = (b_sign != c_sign) && (a_sign == d_sign);
+      if (i_required != i_sign)
+         result.imag_data().negate();
    }
 }
 template <class Backend, class T>
@@ -357,7 +445,7 @@ inline typename std::enable_if< !boost::multiprecision::detail::is_complex<Resul
    using default_ops::eval_is_zero;
    if (!eval_is_zero(val.imag_data()))
    {
-      BOOST_THROW_EXCEPTION(std::runtime_error("Could not convert imaginary number to scalar."));
+      BOOST_MP_THROW_EXCEPTION(std::runtime_error("Could not convert imaginary number to scalar."));
    }
    eval_convert_to(result, val.real_data());
 }
@@ -388,43 +476,43 @@ inline void eval_sqrt(complex_adaptor<Backend>& result, const complex_adaptor<Ba
 
    if (eval_is_zero(val.imag_data()) && (eval_get_sign(val.real_data()) >= 0))
    {
-      constexpr const typename std::tuple_element<0, typename Backend::unsigned_types>::type zero = 0u;
+      constexpr typename std::tuple_element<0, typename Backend::unsigned_types>::type zero = 0u;
       eval_sqrt(result.real_data(), val.real_data());
       result.imag_data() = zero;
       return;
    }
 
-   const bool __my_real_part_is_neg(eval_get_sign(val.real_data()) < 0);
+   const bool my_real_part_is_neg(eval_get_sign(val.real_data()) < 0);
 
-   Backend __my_real_part_fabs(val.real_data());
-   if (__my_real_part_is_neg)
-      __my_real_part_fabs.negate();
+   Backend my_real_part_fabs(val.real_data());
+   if (my_real_part_is_neg)
+      my_real_part_fabs.negate();
 
-   Backend t, __my_sqrt_part;
-   eval_abs(__my_sqrt_part, val);
-   eval_add(__my_sqrt_part, __my_real_part_fabs);
-   eval_ldexp(t, __my_sqrt_part, -1);
-   eval_sqrt(__my_sqrt_part, t);
+   Backend t, my_sqrt_part;
+   eval_abs(my_sqrt_part, val);
+   eval_add(my_sqrt_part, my_real_part_fabs);
+   eval_ldexp(t, my_sqrt_part, -1);
+   eval_sqrt(my_sqrt_part, t);
 
-   if (__my_real_part_is_neg == false)
+   if (my_real_part_is_neg == false)
    {
-      eval_ldexp(t, __my_sqrt_part, 1);
+      eval_ldexp(t, my_sqrt_part, 1);
       eval_divide(result.imag_data(), val.imag_data(), t);
-      result.real_data() = __my_sqrt_part;
+      result.real_data() = my_sqrt_part;
    }
    else
    {
-      const bool __my_imag_part_is_neg(eval_get_sign(val.imag_data()) < 0);
+      const bool my_imag_part_is_neg(eval_get_sign(val.imag_data()) < 0);
 
-      Backend __my_imag_part_fabs(val.imag_data());
-      if (__my_imag_part_is_neg)
-         __my_imag_part_fabs.negate();
+      Backend my_imag_part_fabs(val.imag_data());
+      if (my_imag_part_is_neg)
+         my_imag_part_fabs.negate();
 
-      eval_ldexp(t, __my_sqrt_part, 1);
-      eval_divide(result.real_data(), __my_imag_part_fabs, t);
-      if (__my_imag_part_is_neg)
-         __my_sqrt_part.negate();
-      result.imag_data() = __my_sqrt_part;
+      eval_ldexp(t, my_sqrt_part, 1);
+      eval_divide(result.real_data(), my_imag_part_fabs, t);
+      if (my_imag_part_is_neg)
+         my_sqrt_part.negate();
+      result.imag_data() = my_sqrt_part;
    }
 }
 
@@ -712,13 +800,13 @@ inline void eval_atan(complex_adaptor<Backend>& result, const complex_adaptor<Ba
    using default_ops::eval_log;
    using default_ops::eval_subtract;
 
-   complex_adaptor<Backend> __my_z_times_i, t1, t2, t3;
-   assign_components(__my_z_times_i, arg.imag_data(), arg.real_data());
-   __my_z_times_i.real_data().negate();
+   complex_adaptor<Backend> my_z_times_i, t1, t2, t3;
+   assign_components(my_z_times_i, arg.imag_data(), arg.real_data());
+   my_z_times_i.real_data().negate();
 
-   eval_add(t1, __my_z_times_i, one);
+   eval_add(t1, my_z_times_i, one);
    eval_log(t2, t1);
-   eval_subtract(t1, one, __my_z_times_i);
+   eval_subtract(t1, one, my_z_times_i);
    eval_log(t3, t1);
    eval_subtract(t1, t3, t2);
 
@@ -796,15 +884,15 @@ inline void eval_acosh(complex_adaptor<Backend>& result, const complex_adaptor<B
    using default_ops::eval_multiply;
    using default_ops::eval_subtract;
 
-   complex_adaptor<Backend> __my_zp(arg);
-   eval_add(__my_zp.real_data(), one);
-   complex_adaptor<Backend> __my_zm(arg);
-   eval_subtract(__my_zm.real_data(), one);
+   complex_adaptor<Backend> my_zp(arg);
+   eval_add(my_zp.real_data(), one);
+   complex_adaptor<Backend> my_zm(arg);
+   eval_subtract(my_zm.real_data(), one);
 
    complex_adaptor<Backend> t1, t2;
-   eval_divide(t1, __my_zm, __my_zp);
+   eval_divide(t1, my_zm, my_zp);
    eval_sqrt(t2, t1);
-   eval_multiply(t2, __my_zp);
+   eval_multiply(t2, my_zp);
    eval_add(t2, arg);
    eval_log(result, t2);
 }
@@ -905,8 +993,6 @@ inline std::size_t hash_value(const complex_adaptor<Backend>& val)
 
 } // namespace backends
 
-using boost::multiprecision::backends::complex_adaptor;
-
 template <class Backend>
 struct number_category<complex_adaptor<Backend> > : public std::integral_constant<int, boost::multiprecision::number_kind_complex>
 {};
@@ -927,7 +1013,20 @@ namespace detail {
    template <class Backend>
    struct is_variable_precision<complex_adaptor<Backend> > : public is_variable_precision<Backend>
    {};
-} // namespace detail
+#ifdef BOOST_HAS_INT128
+   template <class Backend>
+   struct is_convertible_arithmetic<int128_type, complex_adaptor<Backend> > : is_convertible_arithmetic<int128_type, Backend>
+   {};
+   template <class Backend>
+   struct is_convertible_arithmetic<uint128_type, complex_adaptor<Backend> > : is_convertible_arithmetic<uint128_type, Backend>
+   {};
+#endif
+#ifdef BOOST_HAS_FLOAT128
+   template <class Backend>
+   struct is_convertible_arithmetic<float128_type, complex_adaptor<Backend> > : is_convertible_arithmetic<float128_type, Backend>
+   {};
+#endif
+   } // namespace detail
 
 
 
