@@ -1,5 +1,5 @@
 // Copyright 2014 Renato Tegon Forti, Antony Polukhin.
-// Copyright 2015-2021 Antony Polukhin.
+// Copyright Antony Polukhin, 2015-2025.
 //
 // Distributed under the Boost Software License, Version 1.0.
 // (See accompanying file LICENSE_1_0.txt
@@ -13,7 +13,7 @@
 
 int main(int argc, char* argv[]) {
     /*<-*/ b2_workarounds::argv_to_path_guard guard(argc, argv); /*->*/
-    boost::shared_ptr<my_refcounting_api> plugin = get_plugin(
+    std::shared_ptr<my_refcounting_api> plugin = get_plugin(
         boost::dll::fs::path(argv[1]) / "refcounting_plugin",
         "create_refc_plugin"
     );
