@@ -10,6 +10,7 @@
 // Test that header file is self-contained.
 #include <boost/json/string.hpp>
 
+#include <boost/core/detail/static_assert.hpp>
 #include <boost/json/monotonic_resource.hpp>
 #include <boost/json/parse.hpp>
 
@@ -22,10 +23,11 @@
 #include "test.hpp"
 #include "test_suite.hpp"
 
-BOOST_JSON_NS_BEGIN
+namespace boost {
+namespace json {
 
-BOOST_STATIC_ASSERT( std::is_nothrow_destructible<string>::value );
-BOOST_STATIC_ASSERT( std::is_nothrow_move_constructible<string>::value );
+BOOST_CORE_STATIC_ASSERT( std::is_nothrow_destructible<string>::value );
+BOOST_CORE_STATIC_ASSERT( std::is_nothrow_move_constructible<string>::value );
 
 class string_test
 {
@@ -539,6 +541,16 @@ public:
                     *s.storage() !=
                     *s2.storage());
             });
+
+            // self
+
+            fail_loop([&](storage_ptr const& sp)
+            {
+                string s1(t.v1, sp);
+                string& s2 = s1;
+                s1 = std::move(s2);
+                BOOST_TEST(s1 == t.v1);
+            });
         }
 
         // operator=(char const*)
@@ -1027,8 +1039,7 @@ public:
             s2.at(1) = 'B';
             BOOST_TEST(s2.at(1) == 'B');
 
-            BOOST_TEST_THROWS(s1.at(s2.size()),
-                std::out_of_range);
+            BOOST_TEST_THROWS_WITH_LOCATION( s1.at(s2.size()) );
         }
 
         // at(size_type) const
@@ -1036,8 +1047,38 @@ public:
             BOOST_TEST(cs1.at(1) == 'b');
             BOOST_TEST(cs2.at(1) == 'B');
 
-            BOOST_TEST_THROWS(cs1.at(cs2.size()),
-                std::out_of_range);
+            BOOST_TEST_THROWS_WITH_LOCATION( cs1.at(cs2.size()) );
+        }
+
+        // try_at(size_type)
+        {
+            s1 = t.v1;
+            s2 = t.v2;
+            BOOST_TEST( *s1.try_at(1) == 'b' );
+            *s1.try_at(1) = '*';
+            BOOST_TEST( *s1.try_at(1) == '*' );
+            *s1.try_at(1) = 'b';
+            BOOST_TEST( *s1.try_at(1) == 'b' );
+
+            BOOST_TEST( *s2.try_at(1) == 'B' );
+            *s2.try_at(1) = '*';
+            BOOST_TEST( *s2.try_at(1) == '*' );
+            *s2.try_at(1) = 'B';
+            BOOST_TEST( *s2.try_at(1) == 'B' );
+
+            system::error_code const ec = s1.try_at(s2.size()).error();
+            BOOST_TEST( ec == error::out_of_range );
+            BOOST_TEST( ec.has_location() );
+        }
+
+        // try_at(size_type) const
+        {
+            BOOST_TEST( *cs1.try_at(1) == 'b' );
+            BOOST_TEST( *cs2.try_at(1) == 'B' );
+
+            system::error_code const ec = s1.try_at(s2.size()).error();
+            BOOST_TEST( ec == error::out_of_range );
+            BOOST_TEST( ec.has_location() );
         }
 
         // operator[&](size_type)
@@ -1134,10 +1175,8 @@ public:
 
         // operator string_view()
         {
-            BOOST_TEST(
-                string_view(cs1) == t.v1);
-            BOOST_TEST(
-                string_view(cs2) == t.v2);
+            BOOST_TEST(cs1.subview() == t.v1);
+            BOOST_TEST(cs2.subview() == t.v2);
         }
     }
 
@@ -1382,17 +1421,15 @@ public:
             // pos out of range
             {
                 string s(t.v1);
-                BOOST_TEST_THROWS(
-                    (s.insert(s.size() + 2, 1, '*')),
-                    std::out_of_range);
+                BOOST_TEST_THROWS_WITH_LOCATION(
+                    (s.insert(s.size() + 2, 1, '*')) );
             }
 
             // size > max_size
             {
                 string s(t.v1);
-                BOOST_TEST_THROWS(
-                    (s.insert(1, s.max_size(), 'a')),
-                    std::length_error);
+                BOOST_TEST_THROWS_WITH_LOCATION(
+                    s.insert(1, s.max_size(), 'a') );
             }
         }
 
@@ -1417,9 +1454,8 @@ public:
             // pos out of range
             {
                 string s(t.v1);
-                BOOST_TEST_THROWS(
-                    (s.insert(s.size() + 2, "*")),
-                    std::out_of_range);
+                BOOST_TEST_THROWS_WITH_LOCATION(
+                    (s.insert(s.size() + 2, "*")));
             }
         }
 
@@ -1639,9 +1675,8 @@ public:
             // pos out of range
             {
                 string s(t.v1);
-                BOOST_TEST_THROWS(
-                    (s.insert(s.size() + 2, "*")),
-                    std::out_of_range);
+                BOOST_TEST_THROWS_WITH_LOCATION(
+                    (s.insert(s.size() + 2, "*")));
             }
         }
 
@@ -1748,9 +1783,8 @@ public:
 
             {
                 string s(t.v1);
-                BOOST_TEST_THROWS(
-                    (s.erase(t.v1.size() + 1, 1)),
-                    std::out_of_range);
+                BOOST_TEST_THROWS_WITH_LOCATION(
+                    (s.erase(t.v1.size() + 1, 1)));
             }
         }
 
@@ -2166,8 +2200,8 @@ public:
             fail_loop([&](storage_ptr const& sp)
             {
                 string s(t.v2, sp);
-                BOOST_TEST_THROWS(s.replace(s.size() + 1, 1, t.v2),
-                    std::out_of_range);
+                BOOST_TEST_THROWS_WITH_LOCATION(
+                    s.replace(s.size() + 1, 1, t.v2));
             });
 
             // outside, shrink
@@ -2389,8 +2423,8 @@ public:
             fail_loop([&](storage_ptr const& sp)
             {
                 string s(t.v2, sp);
-                BOOST_TEST_THROWS(s.replace(s.size() + 1, 1, 1, 'a'),
-                    std::out_of_range);
+                BOOST_TEST_THROWS_WITH_LOCATION(
+                    s.replace(s.size() + 1, 1, 1, 'a'));
             });
         }
 
@@ -2517,35 +2551,39 @@ public:
     {
         test_vectors const t;
 
-        // swap
+        fail_loop([&](storage_ptr const& sp)
         {
-            fail_loop([&](storage_ptr const& sp)
-            {
-                string s1(t.v1, sp);
-                string s2(t.v2, sp);
-                s1.swap(s2);
-                BOOST_TEST(s1 == t.v2);
-                BOOST_TEST(s2 == t.v1);
-            });
+            string s1(t.v1, sp);
+            string s2(t.v2, sp);
+            s1.swap(s2);
+            BOOST_TEST(s1 == t.v2);
+            BOOST_TEST(s2 == t.v1);
+        });
 
-            fail_loop([&](storage_ptr const& sp)
-            {
-                string s1(t.v1, sp);
-                string s2(t.v2, sp);
-                swap(s1, s2);
-                BOOST_TEST(s1 == t.v2);
-                BOOST_TEST(s2 == t.v1);
-            });
+        fail_loop([&](storage_ptr const& sp)
+        {
+            string s1(t.v1, sp);
+            string s2(t.v2, sp);
+            swap(s1, s2);
+            BOOST_TEST(s1 == t.v2);
+            BOOST_TEST(s2 == t.v1);
+        });
 
-            fail_loop([&](storage_ptr const& sp)
-            {
-                string s1(t.v1);
-                string s2(t.v2, sp);
-                s1.swap(s2);
-                BOOST_TEST(s1 == t.v2);
-                BOOST_TEST(s2 == t.v1);
-            });
-        }
+        fail_loop([&](storage_ptr const& sp)
+        {
+            string s1(t.v1);
+            string s2(t.v2, sp);
+            s1.swap(s2);
+            BOOST_TEST(s1 == t.v2);
+            BOOST_TEST(s2 == t.v1);
+        });
+
+        fail_loop([&](storage_ptr const& sp)
+        {
+            string s(t.v1, sp);
+            swap(s, s);
+            BOOST_TEST(s == t.v1);
+        });
     }
 
     void
@@ -2758,13 +2796,13 @@ public:
         {
             std::unordered_set<string>(
                 0,
-                std::hash<string>(32));
+                std::hash<string>());
         }
 #endif
         {
-            std::hash<string> h1(32);
+            std::hash<string> h1;
             std::hash<string> h2(h1);
-            std::hash<string> h3(59);
+            std::hash<string> h3;
             h1 = h3;
             h2 = h3;
             (void)h2;
@@ -2811,4 +2849,5 @@ public:
 
 TEST_SUITE(string_test, "boost.json.string");
 
-BOOST_JSON_NS_END
+} // namespace json
+} // namespace boost
