@@ -1,5 +1,4 @@
-// Copyright (c) 2018-2021 Emil Dotchevski and Reverge Studios, Inc.
-
+// Copyright 2018-2024 Emil Dotchevski and Reverge Studios, Inc.
 // Distributed under the Boost Software License, Version 1.0. (See accompanying
 // file LICENSE_1_0.txt or copy at http://www.boost.org/LICENSE_1_0.txt)
 
@@ -40,7 +39,7 @@ int main()
             r1.error(),
             []( info<1> x ) -> leaf::result<int>
             {
-                BOOST_TEST(x.value==1);
+                BOOST_TEST_EQ(x.value, 1);
                 return 1;
             },
             [&r1]
