@@ -12,7 +12,7 @@
 #  pragma warning (disable : 4310) // cast truncates constant value.
 #endif
 
-#ifdef SYCL_LANGUAGE_VERSION
+#ifdef BOOST_MATH_ENABLE_SYCL
 #include "sycl/sycl.hpp"
 #endif
 
